@@ -3,7 +3,7 @@
  *
  * Written by Russ Allbery <rra@stanford.edu>
  * Based on prior work by Anton Ushakov
- * Copyright 2002, 2003, 2004, 2005, 2006, 2007, 2008
+ * Copyright 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
  *     Board of Trustees, Leland Stanford Jr. University
  *
  * See LICENSE for licensing terms.
@@ -15,20 +15,20 @@
 #include <config.h>
 #include <portable/gssapi.h>
 #include <portable/macros.h>
+#include <portable/socket.h>
 #include <portable/stdbool.h>
+#include <sys/types.h>
 
 /* Forward declaration to avoid unnecessary includes. */
 struct iovec;
 
-BEGIN_DECLS
-
 /* Private structure that holds the details of an open remctl connection. */
 struct remctl {
     const char *host;           /* From remctl_open, stored here because */
     unsigned short port;        /*   remctl v1 requires opening a new    */
     const char *principal;      /*   connection for each command.        */
     int protocol;               /* Protocol version. */
-    int fd;
+    socket_type fd;
     gss_ctx_id_t context;
     char *error;
     struct remctl_output *output;
@@ -36,6 +36,8 @@ struct remctl {
     bool ready;                 /* If true, we are expecting server output. */
 };
 
+BEGIN_DECLS
+
 /* Internal functions should all default to hidden visibility. */
 #pragma GCC visibility push(hidden)
 
