@@ -6,7 +6,7 @@
  *
  * Written by Russ Allbery <rra@stanford.edu>
  * Based on work by Anton Ushakov
- * Copyright 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
+ * Copyright 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
  *     Board of Trustees, Leland Stanford Jr. University
  *
  * See LICENSE for licensing terms.
@@ -25,7 +25,10 @@
 #include <sys/wait.h>
 
 #include <server/internal.h>
-#include <util/util.h>
+#include <util/fdflag.h>
+#include <util/messages.h>
+#include <util/protocol.h>
+#include <util/xmalloc.h>
 
 /* Data structure used to hold details about a running process. */
 struct process {
@@ -229,6 +232,38 @@ fail:
 }
 
 
+/*
+ * Given a configuration line, a command, and a subcommand, return true if
+ * that command and subcommand match that configuration line and false
+ * otherwise.  Handles the ALL and NULL special cases.
+ *
+ * Empty commands are not yet supported by the rest of the code, but this
+ * function copes in case that changes later.
+ */
+static bool
+line_matches(struct confline *cline, const char *command,
+             const char *subcommand)
+{
+    bool okay = false;
+
+    if (strcmp(cline->command, "ALL") == 0)
+        okay = true;
+    if (command != NULL && strcmp(cline->command, command) == 0)
+        okay = true;
+    if (command == NULL && strcmp(cline->command, "EMPTY") == 0)
+        okay = true;
+    if (okay) {
+        if (strcmp(cline->subcommand, "ALL") == 0)
+            return true;
+        if (subcommand != NULL && strcmp(cline->subcommand, subcommand) == 0)
+            return true;
+        if (subcommand == NULL && strcmp(cline->subcommand, "EMPTY") == 0)
+            return true;
+    }
+    return false;
+}
+
+
 /*
  * Process an incoming command.  Check the configuration files and the ACL
  * file, and if appropriate, forks off the command.  Takes the argument vector
@@ -301,13 +336,9 @@ server_run_command(struct client *client, struct config *config,
      */
     for (i = 0; i < config->count; i++) {
         cline = config->rules[i];
-        if (strcmp(cline->command, command) == 0) {
-            if (strcmp(cline->subcommand, "ALL") == 0
-                || (subcommand != NULL
-                    && strcmp(cline->subcommand, subcommand) == 0)) {
-                path = cline->program;
-                break;
-            }
+        if (line_matches(cline, command, subcommand)) {
+            path = cline->program;
+            break;
         }
     }
 
@@ -321,7 +352,8 @@ server_run_command(struct client *client, struct config *config,
         if (argv[i + 1] == NULL && cline->stdin_arg == -1)
             continue;
         if (memchr(argv[i]->iov_base, '\0', argv[i]->iov_len)) {
-            notice("argument %d from user %s contains nul octet", i, user);
+            notice("argument %lu from user %s contains nul octet",
+                   (unsigned long) i, user);
             server_send_error(client, ERROR_BAD_COMMAND,
                               "Invalid command token");
             goto done;
@@ -452,9 +484,9 @@ server_run_command(struct client *client, struct config *config,
             close(fd);
 
         /*
-         * Put the authenticated principal and other connection information in
-         * the environment.  REMUSER is for backwards compatibility with
-         * earlier versions of remctl.
+         * Put the authenticated principal and other connection and command
+         * information in the environment.  REMUSER is for backwards
+         * compatibility with earlier versions of remctl.
          */
         if (setenv("REMUSER", client->user, 1) < 0) {
             syswarn("cannot set REMUSER in environment");
@@ -474,6 +506,10 @@ server_run_command(struct client *client, struct config *config,
                 exit(-1);
             }
         }
+        if (setenv("REMCTL_COMMAND", command, 1) < 0) {
+            syswarn("cannot set REMCTL_COMMAND in environment");
+            exit(-1);
+        }
 
         /* Run the command. */
         execv(path, req_argv);
