@@ -2,7 +2,8 @@
  * Test suite for over-large commands.
  *
  * Written by Russ Allbery <rra@stanford.edu>
- * Copyright 2007, 2009 Board of Trustees, Leland Stanford Jr. University
+ * Copyright 2007, 2009, 2010
+ *     Board of Trustees, Leland Stanford Jr. University
  *
  * See LICENSE for licensing terms.
  */
@@ -22,7 +23,9 @@
 #include <tests/tap/basic.h>
 #include <tests/tap/kerberos.h>
 #include <tests/tap/remctl.h>
-#include <util/util.h>
+#include <util/concat.h>
+#include <util/protocol.h>
+#include <util/xmalloc.h>
 
 
 int
