@@ -2,7 +2,7 @@
  * Test suite for errors returned by the server.
  *
  * Written by Russ Allbery <rra@stanford.edu>
- * Copyright 2006, 2007, 2009
+ * Copyright 2006, 2007, 2009, 2010
  *     Board of Trustees, Leland Stanford Jr. University
  *
  * See LICENSE for licensing terms.
@@ -19,7 +19,10 @@
 #include <tests/tap/basic.h>
 #include <tests/tap/kerberos.h>
 #include <tests/tap/remctl.h>
-#include <util/util.h>
+#include <util/concat.h>
+#include <util/messages.h>
+#include <util/protocol.h>
+#include <util/xmalloc.h>
 
 
 /*
@@ -34,6 +37,8 @@ test_error(struct remctl *r, const char *arg)
     const char *command[] = { "test", NULL, NULL };
 
     command[1] = arg;
+    if (arg == NULL)
+        arg = "(null)";
     if (!remctl_command(r, command)) {
         notice("# remctl error %s", remctl_error(r));
         return ERROR_INTERNAL;
