@@ -2,7 +2,8 @@
  * Test suite for streaming data from the server.
  *
  * Written by Russ Allbery <rra@stanford.edu>
- * Copyright 2006, 2009 Board of Trustees, Leland Stanford Jr. University
+ * Copyright 2006, 2009, 2010
+ *     Board of Trustees, Leland Stanford Jr. University
  *
  * See LICENSE for licensing terms.
  */
@@ -18,7 +19,7 @@
 #include <tests/tap/basic.h>
 #include <tests/tap/kerberos.h>
 #include <tests/tap/remctl.h>
-#include <util/util.h>
+#include <util/concat.h>
 
 
 int
