@@ -2,7 +2,7 @@
  * network test suite.
  *
  * Written by Russ Allbery <rra@stanford.edu>
- * Copyright 2009 Board of Trustees, Leland Stanford Jr. University
+ * Copyright 2009, 2010 Board of Trustees, Leland Stanford Jr. University
  * Copyright (c) 2004, 2005, 2006, 2007
  *     by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1991, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
@@ -20,7 +20,9 @@
 #include <sys/wait.h>
 
 #include <tests/tap/basic.h>
-#include <util/util.h>
+#include <util/macros.h>
+#include <util/messages.h>
+#include <util/network.h>
 
 /* Set this globally to 0 if IPv6 is available but doesn't work. */
 static int ipv6 = 1;
@@ -31,7 +33,7 @@ static int ipv6 = 1;
  * client is supposed to print.
  */
 static void
-listener(int fd)
+listener(socket_type fd)
 {
     int client;
     FILE *out;
@@ -39,14 +41,15 @@ listener(int fd)
 
     client = accept(fd, NULL, NULL);
     close(fd);
-    if (client < 0) {
-        sysnotice("# cannot accept connection from socket");
+    if (client == INVALID_SOCKET) {
+        sysdiag("cannot accept connection from socket");
         ok_block(2, 0, "...socket read test");
+        return;
     }
     ok(1, "...socket accept");
     out = fdopen(client, "r");
     if (fgets(buffer, sizeof(buffer), out) == NULL) {
-        sysnotice("# cannot read from socket");
+        sysdiag("cannot read from socket");
         ok(0, "...socket read");
     }
     is_string("socket test\r\n", buffer, "...socket read");
@@ -62,11 +65,11 @@ listener(int fd)
 static void
 client(const char *host, const char *source)
 {
-    int fd;
+    socket_type fd;
     FILE *out;
 
     fd = network_connect_host(host, 11119, source);
-    if (fd < 0)
+    if (fd == INVALID_SOCKET)
         sysdie("connect failed");
     out = fdopen(fd, "w");
     if (out == NULL)
@@ -85,14 +88,14 @@ client(const char *host, const char *source)
 static void
 test_ipv4(const char *source)
 {
-    int fd;
+    socket_type fd;
     pid_t child;
 
     fd = network_bind_ipv4("127.0.0.1", 11119);
-    if (fd < 0)
+    if (fd == INVALID_SOCKET)
         sysbail("cannot create or bind socket");
     if (listen(fd, 1) < 0) {
-        sysnotice("# cannot listen to socket");
+        sysdiag("cannot listen to socket");
         ok_block(3, 0, "IPv4 server test");
     } else {
         ok(1, "IPv4 server test");
@@ -118,20 +121,21 @@ test_ipv4(const char *source)
 static void
 test_ipv6(const char *source)
 {
-    int fd;
+    socket_type fd;
     pid_t child;
 
     fd = network_bind_ipv6("::1", 11119);
-    if (fd < 0) {
+    if (fd == INVALID_SOCKET) {
         if (errno == EAFNOSUPPORT || errno == EPROTONOSUPPORT
             || errno == EADDRNOTAVAIL) {
             ipv6 = 0;
             skip_block(3, "IPv6 not supported");
+            return;
         } else
             sysbail("cannot create socket");
     }
     if (listen(fd, 1) < 0) {
-        sysnotice("# cannot listen to socket");
+        sysdiag("cannot listen to socket");
         ok_block(3, 0, "IPv6 server test");
     } else {
         ok(1, "IPv6 server test");
@@ -161,9 +165,10 @@ test_ipv6(const char *source UNUSED)
  * use for client connections.
  */
 static void
-test_all(const char *source_ipv4, const char *source_ipv6)
+test_all(const char *source_ipv4, const char *source_ipv6 UNUSED)
 {
-    int *fds, count, fd, i;
+    socket_type *fds, fd;
+    int count, i;
     pid_t child;
     struct sockaddr_storage saddr;
     socklen_t size = sizeof(saddr);
@@ -172,13 +177,13 @@ test_all(const char *source_ipv4, const char *source_ipv6)
     if (count == 0)
         sysbail("cannot create or bind socket");
     if (count > 2) {
-        notice("# got more than two sockets, using just the first two");
+        diag("got more than two sockets, using just the first two");
         count = 2;
     }
     for (i = 0; i < count; i++) {
         fd = fds[i];
         if (listen(fd, 1) < 0) {
-            sysnotice("# cannot listen to socket %d", fd);
+            sysdiag("cannot listen to socket %d", fd);
             ok_block(3, 0, "all address server test");
         } else {
             ok(1, "all address server test (part %d)", i);
@@ -216,14 +221,14 @@ test_all(const char *source_ipv4, const char *source_ipv6)
 static void
 test_create_ipv4(const char *source)
 {
-    int fd;
+    socket_type fd;
     pid_t child;
 
     fd = network_bind_ipv4("127.0.0.1", 11119);
-    if (fd < 0)
+    if (fd == INVALID_SOCKET)
         sysbail("cannot create or bind socket");
     if (listen(fd, 1) < 0) {
-        sysnotice("# cannot listen to socket");
+        sysdiag("cannot listen to socket");
         ok_block(3, 0, "IPv4 network client");
     } else {
         ok(1, "IPv4 network client");
@@ -235,7 +240,7 @@ test_create_ipv4(const char *source)
             FILE *out;
 
             fd = network_client_create(PF_INET, SOCK_STREAM, source);
-            if (fd < 0)
+            if (fd == INVALID_SOCKET)
                 _exit(1);
             sin.sin_family = AF_INET;
             sin.sin_port = htons(11119);
